#
# R script for preprocessing EVI/LSWI indices for the
# Vegetation Photosynthesis and Respiration Model (VPRM)
# obtained from NASA MODIS product (MOD09A1).
#
#
#
# This version of the script is supposed to be used with the STILT model.
#
#
#
# written by
# Christoph Gerbig (CHG), MPI BGC Jena, Germany (cgerbig@bgc-jena.mpg.de)
# Roberto Kretschmer (ROK), MPI BGC Jena, Germany (rkretsch@bgc-jena.mpg.de)
#
#
#
# References:
#
# Lin, J.C., C. Gerbig, S.C. Wofsy, A.E. Andrews, B.C. Daube, K.J. Davis,
# and C.A. Grainger, A near-field tool for simulating the upstream influence
# of atmospheric observations: The Stochastic Time-Inverted Lagrangian Transport
# (STILT) model, Journal of Geophysical Research-Atmospheres, 108 (D16), 2003. 
#
# Mahadevan, P., S. Wofsy, D. M. Matross, X. Xiao, A. Dunn, J. C. Lin, C.
# Gerbig, J. W. Munger, V. Y. Chow, and E. Gottlieb, A Satellite-Based
# Biosphere Parameterization for Net Ecosystem CO2 Exchange: Vegetation
# Photosynthesis and Respiration Model (VPRM), Global Biogeochem. Cycles,
# 22, GB2005, doi:10.1029/2006GB002735, 2008.
#
#




scriptpath <- getwd() # path of preprocessor
sourcepath <-paste(scriptpath,"/RSources/",sep="")

source(paste(scriptpath,"/RSources/monthdayyear.r",sep=""))

# read configuration
source(paste(scriptpath,"/config.r",sep=""))

ftdata   <- TRUE          # shall we connect to ftp and check for data?
projtype <- "GEO"          # type of projection LCC xor GEO
datp     <- rdataout
mdy      <- month.day.year
attachn  <- function(what, pos = 2){if(!(paste("file:",what,sep="")%in%search()))attach(what,pos)}
go       <- graphics.off

# set synmap resolution
spacing <- 1/120

# set number of used vegetation classes
nveg <- 8 

# create working directories
if (!file.exists(mpath)) dir.create(mpath,recursive=TRUE)
if (!file.exists(rdatain)) dir.create(rdatain,recursive=TRUE)
if (!file.exists(rdataout)) dir.create(rdataout,recursive=TRUE)

# check for synmap
if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
  print("SYNMAP not there. Prepare one...")
  source(paste(scriptpath,"/RSources/getSynmap.r",sep=""))
  if (!file.exists(paste(rdatain,"/.RDatasynmap.",synmaptouse,sep=""))) {
   stop("Could not prepare SYNMAP!")
  }
}

setwd(rdataout)

library("h5r")

source(paste(scriptpath,"/RSources/assignr.r",sep=""))
source(paste(scriptpath,"/RSources/project.nice.r",sep=""))
source(paste(scriptpath,"/RSources/getminmax.r",sep=""))
source(paste(scriptpath,"/RSources/distance.r",sep=""))
source(paste(scriptpath,"/RSources/getr.r",sep=""))
source(paste(scriptpath,"/RSources/loess.r",sep=""))
source(paste(scriptpath,"/RSources/imagell.r",sep=""))
source(paste(scriptpath,"/RSources/na_replace.r",sep=""))
source(paste(scriptpath,"/RSources/combineTiles.r",sep=""))
source(paste(scriptpath,"/RSources/hdfread.r",sep=""))
source(paste(scriptpath,"/RSources/parseNamelist.r",sep=""))
source(paste(scriptpath,"/RSources/getModis.r",sep=""))
source(paste(scriptpath,"/RSources/extractAndReduce.r",sep=""))
source(paste(scriptpath,"/RSources/doResampling.r",sep=""))
source(paste(scriptpath,"/RSources/readIndicesIntoR.r",sep=""))
source(paste(scriptpath,"/RSources/cutToDomain.r",sep=""))
source(paste(scriptpath,"/RSources/doAggregation.r",sep=""))
source(paste(scriptpath,"/RSources/writeOutput.r",sep=""))

# get additional configuration settings
# configuration is saved in a R-object file called ".RDataprojinfo"
parseNamelist()

# load grid configuration
projinfo <- getr("projinfo",datp)

# set 8 day intervals to process
# to do: calculate from start and end date set in namelist!
#doyuse <- doyall   <- seq(as.numeric(paste(projinfo$year,"009",sep="")),as.numeric(paste(projinfo$year,"361",sep="")),by=8)
#doyuse   <- seq(as.numeric(paste(projinfo$year,"041",sep="")),as.numeric(paste(projinfo$year,"305",sep="")),by=8)

# now the full year (tk) 05.07.10
#doyuse <- doyall   <- seq(as.numeric(paste(projinfo$year,"001",sep="")),as.numeric(paste(projinfo$year,"365",sep="")),by=8)
doyuse <- doyall   <- seq(as.numeric(paste(projinfo$year,"001",sep="")),as.numeric(paste(projinfo$year,projinfo$ldoy,sep="")),by=8)
doyuse<-doyuse[doyuse!=2001169] #exclude missing MODIS acquisition period

# cd to working directory
setwd(mpath)

# - check for MODIS data
# - make sure we have all the tiles needed for the configuration
#
# A tile lookup table (TLUT) is created and returned by the function below.
# The TLUT contains all tiles needed for the domain
# and later on also the tile coordinates in lat/lon, calculated during resampling.
# But for now TLUT only contains MODIS tile IDs (e.g. h17v04).
if (!file.exists("TLUT.RData")) TLUT <- getModis(doyuse)
if (file.exists("TLUT.RData")&exists("prepr.additionalTF")){#also need to get MODIS data from rest of year, if only part of year was processed before
  if(prepr.additionalTF){
    system(paste("rm -f ",mpath,"/TLUT.RData",sep=""))
    TLUT <- getModis(doyuse)
  }
}
# check if we have to initialize MODIS fields (read into R)
skip <- TRUE
for (ind in indices) {
 if (length(list.files(path=rdataout,pattern=glob2rx(paste(".*",ind,"*combined",sep="")),all.files=T))==0) skip<-FALSE
}
#skip <- FALSE # is to comment (tk)  05.07.10

#check if full year has been processed, or if only a part if the year has been processed
#if(skip){
#  #get one tile, check length of time dimension
#  checkf<-list.files(path=rdataout,pattern=glob2rx(paste(".*",indices[1],".smooth*",sep="")),all.files=T)[1]
#  checkdim<-dim(getr(substring(checkf,7,nchar(checkf)),path=rdataout))[3]
#  if(length(doyuse)>checkdim) skip<-FALSE
#  file.remove(list.files(path=rdataout,pattern=glob2rx(paste(".*",indices[1],".smooth*",sep="")),all.files=T,full.names=TRUE))
#  file.remove(list.files(path=rdataout,pattern=glob2rx(paste(".*",indices[2],".smooth*",sep="")),all.files=T,full.names=TRUE))
#  system(paste("rm -f ",rdataout,"/.*combined*",sep=""))
#  system(paste("rm -f ",rdataout,"/.*Classed*",sep=""))
#  system(paste("rm -f ",rdataout,"/.*Aggregated*",sep=""))

#}

if (file.exists("TLUT.RData")) load("TLUT.RData")

if (!skip) {
 # delete old resample log
 if ( file.exists("resample.log") )
 system("rm resample.log")

 # Extracting EVI and LSWI and reducing MODIS data to 1km.
 for (ind in indices) extractAndReduce(ind,TLUT,projinfo$year,doyuse) # defined in extractAndReduce.r

 # Resample EVI/LSWI from SIN to GEO (LAT/LON) projection.
 # LL and UR corners are added to TLUT for every tile (needed only once).
 # Adding happens only if resampling is done for the first time. If the resampled files are
 # already there, this step is skipped, so TLUT has to be loaded (see below).
 for (ind in indices) TLUT <- doResampling(ind,TLUT,projinfo) # defined in doResampling.r

 # load old TLUT if existing, else save current one.
 if (file.exists("TLUT.RData")) load("TLUT.RData") else save(TLUT,file="TLUT.RData")

 print(TLUT)

 # convert EVI/LSWI HDF files to R objects.
 for (ind in indices) readIndicesIntoR(ind,projinfo,doyuse) # defined in readIndicesIntoR.r

 gc()
}


###################### SMOOTHING: loess filtering ##############################
# to do: which times to exclude because of snow? (use special MODIS data for SNOW)
# to do: optimize performance
#skip <- FALSE # TEST ONLY

source(paste(scriptpath,"/RSources/loess.r",sep=""))
if (!skip) {
 for (ind in indices) {
  # check if already done
  #if (length(list.files(path=mpath,pattern=glob2rx(paste(ind,"*smooth*.nc",sep=""))))!=0) next
 
  # do it in parallel?
  if (do_parallel) {
   system(paste(scriptpath,"/submit_loess.sh ",scriptpath," '",ind,"'",sep=""))
  } else {
   # do it per tile
   for (i in 1:(length(TLUT)/6)) {
    if ((length(TLUT)/6)==1) {
     if (TLUT[1]<10) H <- paste("0",TLUT[1],sep="") else H <- TLUT[1]
     if (TLUT[2]<10) V <- paste("0",TLUT[2],sep="") else V <- TLUT[2]
    } else {
     if (TLUT[i,1]<10) H <- paste("0",TLUT[i,1],sep="") else H <- TLUT[i,1]
     if (TLUT[i,2]<10) V <- paste("0",TLUT[i,2],sep="") else V <- TLUT[i,2]
    }
    projs<-paste("h",H,"v",V,".",projinfo$year,".ier1km",sep="")
    Smoothevilswi(ind,proj=projs,doyall=doyall,doyuse=doyuse) # defined in loess.r
   }
   rm(projs);gc()
  }
 }
}



############################# combining tiles ##################################
if (!skip) {
 for (ind in indices) combineTiles(ind,TLUT,doyuse,projinfo)  # defined in combineTiles.r
 gc()
}

######## extract evi/lswi indices and vegetation map for target grid ###########
######## gap filling
for (ind in indices) cutToDomain(ind,doyuse,projinfo) # defined in cutToDomain.r
gc()

################# aggregation and projection to target grid ####################
for (ind in indices) doAggregateGEO(ind,doyuse,nveg)   # defined in doAggregation.r
gc()

########## calculate min and max indices #######################################
###### for near-real-time production: not do this
for (ind in indices) getMinMaxIndices(ind,projinfo$year,doyuse) # defined in getminmax.r
gc()


########### write NETCDF files #################################################
# generate output directory 
start_day_of_year<-min(doyuse-projinfo$year*1000)

resdir <- paste(outdir,"/","VPRM_input_",domainname,"_",projinfo$year,sep="")
if (!file.exists(resdir)) dir.create(resdir,recursive=TRUE)

writeVegetationMap(resdir)
for (ind in indices) writeIndices(ind,start_day_of_year,doyuse,resdir)
gc()

file.copy(paste(scriptpath,"/config.r",sep=""),resdir)

######### All done. ############################################################
print("Preprocessing of VPRM indices finished.")
print(date())







# used VPRM classes:
# 1 Trees evergreen
# 2 Trees deciduous
# 3 Trees mixed
# 4 Trees and shrubs
# 5 Trees and grasses
# 6 Trees and crops
# 7 grasses
# 8 Barren, Urban and built-up, Permanent snow and ice
# c("trees evergreen", "trees deciduous", "trees mixed", "trees and shrubs", "trees and grasses", "trees and crops", "grasses", "barren, urban and built-up, permanent snow and ice")

#SYNMAP classes:
#1  Trees Needle Evergreen Trees
#2  Trees Needle Deciduous
#3  Trees Needle Mixed
#4  Trees Broad Evergreen
#5  Trees Broad Deciduous
#6  Trees Broad Mixed
#7  Trees Mixed Evergreen
#8  Trees Mixed Deciduous
#9  Trees Mixed Mixed
#10 Trees and shrubs Needle Evergreen
#11 Trees and shrubs Needle Deciduous
#12 Trees and shrubs Needle Mixed
#13 Trees and shrubs Broad Evergreen
#14 Trees and shrubs Broad Deciduous
#15 Trees and shrubs Broad Mixed
#16 Trees and shrubs Mixed Evergreen
#17 Trees and shrubs Mixed Deciduous
#18 Trees and shrubs Mixed Mixed
#19 Trees and grasses Needle Evergreen
#20 Trees and grasses Needle Deciduous
#21 Trees and grasses Needle Mixed
#22 Trees and grasses Broad Evergreen
#23 Trees and grasses Broad Deciduous
#24 Trees and grasses Broad Mixed
#25 Trees and grasses Mixed Evergreen
#26 Trees and grasses Mixed Deciduous
#27 Trees and grasses Mixed Mixed
#28 Trees and crops Needle Evergreen Crops/natural vegetation mosaic
#29 Trees and crops Needle Deciduous
#30 Trees and crops Needle Mixed
#31 Trees and crops Broad Evergreen
#32 Trees and crops Broad Deciduous
#33 Trees and crops Broad Mixed
#34 Trees and crops Mixed Evergreen
#35 Trees and crops Mixed Deciduous
#36 Trees and crops Mixed Mixed
#37 Shrubs
#38 Shrubs and grasses
#39 Shrubs and crops
#40 Shrubs and barren
#41 Grasses
#42 Grasses and crops
#43 Grasses and barren
#44 Crops
#45 Barren
#46 Urban and built-up
#47 Permanent snow and ice
#48 Water (originally class 0 in SYNMAP)

# MAPPING
#VPRM	Synmap
#1	1
#2	2
#3	3
#1	4
#2	5
#3	6
#1	7
#2	8
#3	9
#4	10
#4	11
#4	12
#4	13
#4	14
#4	15
#4	16
#4	17
#4	18
#5	19
#5	20
#5	21
#5	22
#5	23
#5	24
#5	25
#5	26
#5	27
#6	28
#6	29
#6	30
#6	31
#6	32
#6	33
#6	34
#6	35
#6	36
#4	37
#4	38
#4	39
#4	40
#7	41
#7	42
#7	43
#6	44
#8	45
#8	46
#8	47
#8	48
